/*
Copyright © 2021 da/net Research Group <danet@h-da.de>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package cmd

import (
	apb "code.fbi.h-da.de/danet/gosdn/api/go/gosdn/rbac"
	"code.fbi.h-da.de/danet/gosdn/controller/api"

	log "github.com/sirupsen/logrus"

	"github.com/google/uuid"
	"github.com/pterm/pterm"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

// userUpdateCmd represents the update command.
var userUpdateCmd = &cobra.Command{
	Use:   "update",
	Short: "Updates a user with provided data",
	Long: `Updates a user with provided data.
    User name and password hashed with (add hash method here!) required,
    role is optional but needed to operate on PNDs.`,

	Run: func(cmd *cobra.Command, args []string) {
		existingUser, err := api.GetUser(
			createContextWithAuthorization(),
			viper.GetString("controllerAPIEndpoint"),
			nbUserName,
			uuid.Nil,
		)
		if err != nil {
			pterm.Error.Println(err)
			return
		}

		roles := map[string]string{}
		// only active pnd for now, add option for additional param later
		roles[viper.GetString("CLI_PND")] = nbUserRole

		// only one for now, add option to update more users at once later
		users := []*apb.UpdateUser{
			{
				Id:       nbUserID,
				Metadata: existingUser.GetUser().GetMetadata(),
				Name:     nbUserName,
				Password: nbUserPwd,
				Roles:    roles,
			},
		}

		_, err = api.UpdateUsers(createContextWithAuthorization(), viper.GetString("controllerAPIEndpoint"), users)
		if err != nil {
			pterm.Error.Println(err)
			return
		}
		log.Info("Users updated.")
	},
	PostRun: func(cmd *cobra.Command, args []string) {
		// Necessary for prompt mode. The flag variables have to be resetted,
		// since in prompt mode the program keeps running.
		nbUserID, nbUserName, nbUserPwd, nbUserRole = "", "", "", ""
	},
}

var nbUserID string

func init() {
	userCmd.AddCommand(userUpdateCmd)

	userUpdateCmd.Flags().StringVar(&nbUserID, "i", "", "id of the user")
	userUpdateCmd.Flags().StringVar(&nbUserName, "u", "", "username for login")
	userUpdateCmd.Flags().StringVar(&nbUserPwd, "p", "", "pwd for login")
	userUpdateCmd.Flags().StringVar(&nbUserRole, "r", "", "role for user")
}
