package server

import (
	"context"
	"time"

	apb "code.fbi.h-da.de/danet/gosdn/api/go/gosdn/app"
	"code.fbi.h-da.de/danet/gosdn/controller/app"

	"github.com/bufbuild/protovalidate-go"
	log "github.com/sirupsen/logrus"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

// AppServer represents a AppServiceServer.
type AppServer struct {
	apb.UnimplementedAppServiceServer
	appService     app.ManagementService
	protoValidator *protovalidate.Validator
}

// NewAppServer creates a new AppServiceServer.
func NewAppServer(appService app.ManagementService, protoValidator *protovalidate.Validator) *AppServer {
	return &AppServer{
		appService:     appService,
		protoValidator: protoValidator,
	}
}

// Register checks if the app already exists and if not creates a new one.
func (a *AppServer) Register(ctx context.Context, request *apb.AppRegisterRequest) (*apb.AppRegisterResponse, error) {
	if err := a.protoValidator.Validate(request); err != nil {
		return nil, status.Errorf(codes.Aborted, "%v", err)
	}

	app, err := a.appService.Register(request.GetAppname(), request.GetToken())
	if err != nil {
		return nil, err
	}

	log.Infof("app %s registered", request.GetAppname())

	return &apb.AppRegisterResponse{
		Timestamp:       time.Now().UnixNano(),
		Queueconnection: app.GetCredentials(),
	}, nil
}

// Deregister deregisters an app.
func (a *AppServer) Deregister(ctx context.Context, request *apb.AppDeregisterRequest) (*apb.AppDeregisterResponse, error) {
	if err := a.protoValidator.Validate(request); err != nil {
		return nil, status.Errorf(codes.Aborted, "%v", err)
	}

	err := a.appService.Deregister(request.GetAppname())
	if err != nil {
		return nil, err
	}

	return &apb.AppDeregisterResponse{
		Timestamp: time.Now().UnixNano(),
	}, nil
}
