import { api, PluginInternalServiceAvailablePluginsApiArg } from "@api/api"
import { warnMessage } from "@helper/debug"
import { createAsyncThunk } from "@reduxjs/toolkit"
import { RootState } from "src/stores"
import { setPlugins } from "../reducer/plugin.reducer"

const FETCH_PLUGIN_ACTION = 'subscription/plugin/fetchPlugins'


export const fetchPluginsThunk = createAsyncThunk(
    FETCH_PLUGIN_ACTION,
    async (_, thunkApi) => {
        const { user } = thunkApi.getState() as RootState

        if (!user.user?.roles) {
            warnMessage('Plugin fetch was triggered but user data is not presence')
            return
        }

        const payload: PluginInternalServiceAvailablePluginsApiArg = {
            timestamp: new Date().getTime().toString(),
        }


        const plugins = await thunkApi.dispatch(api.endpoints.pluginInternalServiceAvailablePlugins.initiate(payload))

        if (plugins.error || !plugins.data?.plugins) {
            warnMessage('Plugin fetch returned an error: ' + plugins.error)
            return
        }

        thunkApi.dispatch(
            setPlugins(plugins.data?.plugins)
        )
    }
)